﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using Microsoft.Practices.Prism.ViewModel;

namespace TestLineMarker
{
    public class SeriesData:NotificationObject 
    {
        private readonly List<PointData> _points; 

        public ObservableCollection<PointData> Series { get; set; }

        public ObservableCollection<int> AvailableSelectionOfNumberOfMarkers { get; set; } 

        public int PointsCount { get; set; }


        private bool _isSwapped;

        public bool IsSwapped
        {
            get { return _isSwapped; }
            set
            {
                _isSwapped = value;
                RaisePropertyChanged("IsSwapped");
            }
        }

        private int _numberOfMarkers;

        public int NumberOfMarkers 
        {
            get { return _numberOfMarkers; }
            set
            {
                _numberOfMarkers = value;
                RaisePropertyChanged("NumberOfMarkers");
            }
        }
 
        public SeriesData()
        {
            AvailableSelectionOfNumberOfMarkers = new ObservableCollection<int>() {5, 10, 15, 20};
            var random = new Random();
            PointsCount = 10000;
            NumberOfMarkers = 10;
            _points = new List<PointData>(PointsCount);
            var pointInternal = PointsCount/NumberOfMarkers;
            for (int i = 0; i < PointsCount; i++)
            {
                var loss = (PointsCount - i)*10;
                var probability = i*10.0/PointsCount;
                _points.Add(new PointData
                               {
                                   Loss = loss,
                                   Probability = probability,
                                   UpperBand = loss + random.NextDouble()*PointsCount,
                                   LowerBand = loss - random.NextDouble()*PointsCount,
                                   IsMarkerShown = i % pointInternal == 0
                               });
            }
            Series = new ObservableCollection<PointData>(_points);
        }

        public void Update(double minProbabilityRatio, double maxProbabilityRatio, double minLossRatio, double maxLossRatio)
        {
            var minProbability = minProbabilityRatio*10;
            var maxProbability = maxProbabilityRatio*10;
            var minLoss = PointsCount*10 - maxLossRatio*PointsCount*10;
            var maxLoss = PointsCount*10 - minLossRatio*PointsCount*10;
            var selection =
                Series.Where(
                    item =>
                    item.Probability > minProbability && item.Probability < maxProbability &&
                    item.Loss < maxLoss && item.Loss > minLoss).ToList();
            var count = selection.Count;
            var pointInternal = count / NumberOfMarkers;
            if (count % NumberOfMarkers != 0) pointInternal++;

            for(int i=0;i<selection.Count;i++)
            {
                selection[i].IsMarkerShown = i%pointInternal == 0;
            }
            foreach(var item in Series)
            {
                if (selection.IndexOf(item) >= 0) continue;
                item.IsMarkerShown = false;
            }
            
        }
    }
}
